@@ -23,7 +23,7 @@ use tcgeneric::{
     label, Id, Instance, Label, Map, NativeClass, TCBoxStream, TCBoxTryStream, TCPathBuf, Tuple,
 };
 
-use crate::chain::{ChainType, Subject, CHAIN, NULL_HASH};
+use crate::chain::{ChainType, Subject, BLOCK_SIZE, CHAIN, NULL_HASH};
 use crate::collection::*;
 use crate::fs;
 use crate::route::Public;
@@ -49,8 +49,9 @@ impl History {
     }
 
     pub async fn create(txn_id: TxnId, dir: fs::Dir, class: ChainType) -> TCResult<Self> {
+        let block = ChainBlock::new(NULL_HASH);
         let file: fs::File<ChainBlock> = dir.create_file(txn_id, CHAIN.into(), class).await?;
-        file.create_block(txn_id, 0u64.into(), ChainBlock::new(NULL_HASH))
+        file.create_block(txn_id, 0u64.into(), block, BLOCK_SIZE)
             .await?;
 
         let dir = dir.create_dir(txn_id, DATA.into()).await?;
@@ -102,7 +103,7 @@ impl History {
                     let schema = btree.schema().to_vec();
                     let classpath = BTreeType::default().path();
 
-                    if self.dir.contains(&txn_id, &hash).await? {
+                    if self.dir.contains(txn_id, &hash).await? {
                         debug!("BTree with hash {} is already saved", hash);
                     } else {
                         let file = self
@@ -125,7 +126,7 @@ impl History {
                     let schema = table.schema().clone();
                     let classpath = TableType::default().path();
 
-                    if self.dir.contains(&txn_id, &hash).await? {
+                    if self.dir.contains(txn_id, &hash).await? {
                         debug!("Table with hash {} is already saved", hash);
                     } else {
                         let dir = self.dir.create_dir(txn_id, hash.clone()).await?;
@@ -151,7 +152,7 @@ impl History {
                         Tensor::Dense(dense) => {
                             let hash = dense.hash_hex(&txn).await?.parse()?;
 
-                            if self.dir.contains(&txn_id, &hash).await? {
+                            if self.dir.contains(txn_id, &hash).await? {
                                 debug!("Tensor with hash {} is already saved", hash);
                             } else {
                                 let file = self
@@ -168,7 +169,7 @@ impl History {
                         Tensor::Sparse(sparse) => {
                             let hash = sparse.hash_hex(&txn).await?.parse()?;
 
-                            if self.dir.contains(&txn_id, &hash).await? {
+                            if self.dir.contains(txn_id, &hash).await? {
                                 debug!("Tensor with hash {} is already saved", hash);
                             } else {
                                 let dir = self.dir.create_dir(txn_id, hash.clone()).await?;
@@ -198,17 +199,21 @@ impl History {
         Ok(block.mutations().keys().next().cloned())
     }
 
-    pub async fn latest_block_id(&self, txn_id: &TxnId) -> TCResult<u64> {
+    pub async fn latest_block_id(&self, txn_id: TxnId) -> TCResult<u64> {
         self.latest.read(txn_id).map_ok(|id| *id).await
     }
 
-    pub async fn contains_block(&self, txn_id: &TxnId, block_id: u64) -> TCResult<bool> {
+    pub async fn contains_block(&self, txn_id: TxnId, block_id: u64) -> TCResult<bool> {
         self.file.contains_block(txn_id, &block_id.into()).await
     }
 
-    pub async fn create_next_block(&self, txn_id: TxnId) -> TCResult<fs::Block<ChainBlock>> {
+    pub async fn create_next_block(
+        &self,
+        txn_id: TxnId,
+    ) -> TCResult<<fs::File<ChainBlock> as File<ChainBlock>>::Write> {
         let mut latest = self.latest.write(txn_id).await?;
         let last_block = self.read_block(txn_id, (*latest).into()).await?;
+
         let hash = last_block.hash().await?;
         let block = ChainBlock::new(hash);
 
@@ -216,7 +221,7 @@ impl History {
         debug!("creating next chain block {}", *latest);
 
         self.file
-            .create_block(txn_id, (*latest).into(), block)
+            .create_block(txn_id, (*latest).into(), block, BLOCK_SIZE)
             .await
     }
 
@@ -224,7 +229,7 @@ impl History {
         &self,
         txn_id: TxnId,
         block_id: u64,
-    ) -> TCResult<fs::BlockRead<ChainBlock>> {
+    ) -> TCResult<<fs::File<ChainBlock> as File<ChainBlock>>::Read> {
         self.file.read_block(txn_id, block_id.into()).await
     }
 
@@ -232,23 +237,30 @@ impl History {
         &self,
         txn_id: TxnId,
         block_id: u64,
-    ) -> TCResult<fs::BlockWrite<ChainBlock>> {
+    ) -> TCResult<<fs::File<ChainBlock> as File<ChainBlock>>::Write> {
         self.file.write_block(txn_id, block_id.into()).await
     }
 
-    pub async fn read_latest(&self, txn_id: TxnId) -> TCResult<fs::BlockRead<ChainBlock>> {
-        let latest = self.latest.read(&txn_id).await?;
+    pub async fn read_latest(
+        &self,
+        txn_id: TxnId,
+    ) -> TCResult<<fs::File<ChainBlock> as File<ChainBlock>>::Read> {
+        let latest = self.latest.read(txn_id).await?;
         self.read_block(txn_id, (*latest).into()).await
     }
 
-    pub async fn write_latest(&self, txn_id: TxnId) -> TCResult<fs::BlockWrite<ChainBlock>> {
-        let latest = self.latest.read(&txn_id).await?;
+    pub async fn write_latest(
+        &self,
+        txn_id: TxnId,
+    ) -> TCResult<<fs::File<ChainBlock> as File<ChainBlock>>::Write> {
+        let latest = self.latest.read(txn_id).await?;
         self.write_block(txn_id, (*latest).into()).await
     }
 
     pub async fn apply_last(&self, txn: &Txn, subject: &Subject) -> TCResult<()> {
-        let latest = *self.latest.read(txn.id()).await?;
+        let latest = *self.latest.read(*txn.id()).await?;
         let block = self.read_block(*txn.id(), latest.into()).await?;
+
         let last_block = if latest > 0 && block.mutations().is_empty() {
             self.read_block(*txn.id(), (latest - 1).into()).await?
         } else {
@@ -290,7 +302,7 @@ impl History {
         let txn_id = *txn.id();
 
         let (latest, other_latest) =
-            try_join!(self.latest.read(&txn_id), other.latest.read(&txn_id))?;
+            try_join!(self.latest.read(txn_id), other.latest.read(txn_id))?;
 
         if (*latest) > (*other_latest) {
             return Err(TCError::bad_request(
@@ -302,7 +314,9 @@ impl History {
         const ERR_DIVERGENT: &str = "chain to replicate diverges at block";
         for i in 0u64..*latest {
             let block = self.read_block(txn_id, i.into()).await?;
+
             let other = other.read_block(txn_id, i.into()).await?;
+
             if &*block != &*other {
                 return Err(TCError::bad_request(ERR_DIVERGENT, i));
             }
@@ -311,6 +325,7 @@ impl History {
         let mut i = *latest;
         loop {
             debug!("copy history from block {}", i);
+
             let source = other.read_block(txn_id, i).await?;
             let mut dest = self.write_block(txn_id, i).await?;
 
@@ -351,9 +366,6 @@ impl History {
 
             let (source_hash, dest_hash) = try_join!(source.hash(), dest.hash())?;
             if source_hash != dest_hash {
-                debug!("source {:?}", &*source);
-                debug!("dest {:?}", &*dest);
-
                 return Err(TCError::bad_request(
                     "error replicating chain",
                     format!("hashes diverge at block {}", i),
@@ -362,7 +374,7 @@ impl History {
 
             i += 1;
 
-            if other.contains_block(txn.id(), i).await? {
+            if other.contains_block(*txn.id(), i).await? {
                 self.create_next_block(*txn.id()).await?;
             } else {
                 break;
@@ -420,7 +432,7 @@ impl History {
                 let schema = Value::try_cast_from(schema, |v| schema_err(v))?;
                 let schema = schema.try_cast_into(|v| schema_err(v))?;
 
-                let file = self.dir.get_file(txn.id(), &hash).await?.ok_or_else(|| {
+                let file = self.dir.get_file(*txn.id(), &hash).await?.ok_or_else(|| {
                     TCError::internal(format!("Chain is missing historical state {}", hash))
                 })?;
 
@@ -438,7 +450,7 @@ impl History {
                 let schema = Value::try_cast_from(schema, |v| schema_err(v))?;
                 let schema = schema.try_cast_into(|v| schema_err(v))?;
 
-                let dir = self.dir.get_dir(txn.id(), &hash).await?;
+                let dir = self.dir.get_dir(*txn.id(), &hash).await?;
                 let dir = dir.ok_or_else(|| {
                     TCError::internal(format!("missing historical Chain state {}", hash))
                 })?;
@@ -458,7 +470,7 @@ impl History {
 
                 match tt {
                     TensorType::Dense => {
-                        let file = self.dir.get_file(txn.id(), &hash).await?;
+                        let file = self.dir.get_file(*txn.id(), &hash).await?;
                         let file = file.ok_or_else(|| {
                             TCError::internal(format!("missing historical Chain state {}", hash))
                         })?;
@@ -467,7 +479,7 @@ impl History {
                         Ok(Collection::Tensor(tensor.into()))
                     }
                     TensorType::Sparse => {
-                        let dir = self.dir.get_dir(txn.id(), &hash).await?;
+                        let dir = self.dir.get_dir(*txn.id(), &hash).await?;
                         let dir = dir.ok_or_else(|| {
                             TCError::internal(format!("missing historical Chain state {}", hash))
                         })?;
@@ -497,20 +509,20 @@ impl Persist<fs::Dir> for History {
         let txn_id = txn.id();
 
         let file: fs::File<ChainBlock> = dir
-            .get_file(txn_id, &CHAIN.into())
+            .get_file(*txn_id, &CHAIN.into())
             .await?
             .ok_or_else(|| TCError::internal("Chain has no history file"))?;
 
-        let dir = dir
-            .get_dir(txn_id, &DATA.into())
-            .await?
-            .ok_or_else(|| TCError::internal("Chain has no data directory"))?;
+        // if there's no data in the data dir, it may not have been sync'd to the filesystem
+        // so just create a new one in memory
+        let dir = dir.get_or_create_dir(*txn_id, DATA.into()).await?;
 
         let mut last_hash = Bytes::from(NULL_HASH);
         let mut latest = 0;
 
         loop {
             let block = file.read_block(*txn_id, latest.into()).await?;
+
             if block.last_hash() == &last_hash {
                 last_hash = block.last_hash().clone();
             } else {
@@ -520,7 +532,7 @@ impl Persist<fs::Dir> for History {
                 )));
             }
 
-            if file.contains_block(txn_id, &(latest + 1).into()).await? {
+            if file.contains_block(*txn_id, &(latest + 1).into()).await? {
                 latest += 1;
             } else {
                 break;
@@ -578,7 +590,7 @@ impl de::Visitor for HistoryVisitor {
             .await?;
 
         let first_block = ChainBlock::new(NULL_HASH);
-        file.create_block(txn_id, 0u64.into(), first_block)
+        file.create_block(txn_id, 0u64.into(), first_block, BLOCK_SIZE)
             .map_err(de::Error::custom)
             .await?;
 
@@ -624,9 +636,7 @@ impl de::Visitor for HistoryVisitor {
             let mut block = history
                 .create_next_block(txn_id)
                 .map_err(de::Error::custom)
-                .await?
-                .write()
-                .await;
+                .await?;
 
             if block.last_hash() != &hash {
                 let hash = hex::encode(hash);
@@ -696,7 +706,7 @@ impl<'en> IntoView<'en, fs::Dir> for History {
         debug!("History::into_view");
 
         let txn_id = *txn.id();
-        let latest = self.latest.read(&txn_id).await?;
+        let latest = self.latest.read(txn_id).await?;
 
         let file = self.file.clone();
         let read_block = move |block_id| Box::pin(file.clone().read_block_owned(txn_id, block_id));
