@@ -38,7 +38,7 @@ mod data;
 
 mod sync;
 
-const BLOCK_SIZE: u64 = 1_000_000;
+const BLOCK_SIZE: usize = 1_000_000;
 const CHAIN: Label = label("chain");
 const NULL_HASH: Vec<u8> = vec![];
 const PREFIX: PathLabel = path_label(&["state", "chain"]);
@@ -222,7 +222,7 @@ impl Subject {
                     .create_file(txn_id, SUBJECT.into(), value.class())
                     .await?;
 
-                file.create_block(txn_id, SUBJECT.into(), value.clone())
+                file.create_block(txn_id, SUBJECT.into(), value.clone(), BLOCK_SIZE)
                     .await?;
 
                 Ok(Self::Value(file))
@@ -233,14 +233,14 @@ impl Subject {
     async fn load(txn: &Txn, schema: Schema, dir: &fs::Dir) -> TCResult<Self> {
         match schema {
             Schema::BTree(schema) => {
-                if let Some(file) = dir.get_file(txn.id(), &SUBJECT.into()).await? {
+                if let Some(file) = dir.get_file(*txn.id(), &SUBJECT.into()).await? {
                     BTreeFile::load(txn, schema, file).map_ok(Self::BTree).await
                 } else {
                     Self::create(Schema::BTree(schema), dir, *txn.id()).await
                 }
             }
             Schema::Table(schema) => {
-                if dir.is_empty(txn.id()).await? {
+                if dir.is_empty(*txn.id()).await? {
                     Self::create(Schema::Table(schema), dir, *txn.id()).await
                 } else {
                     TableIndex::load(txn, schema, dir.clone())
@@ -250,7 +250,7 @@ impl Subject {
             }
             #[cfg(feature = "tensor")]
             Schema::Dense(schema) => {
-                if let Some(file) = dir.get_file(txn.id(), &SUBJECT.into()).await? {
+                if let Some(file) = dir.get_file(*txn.id(), &SUBJECT.into()).await? {
                     DenseTensor::load(txn, schema, file)
                         .map_ok(Self::Dense)
                         .await
@@ -260,7 +260,7 @@ impl Subject {
             }
             #[cfg(feature = "tensor")]
             Schema::Sparse(schema) => {
-                if let Some(dir) = dir.get_dir(txn.id(), &SUBJECT.into()).await? {
+                if let Some(dir) = dir.get_dir(*txn.id(), &SUBJECT.into()).await? {
                     SparseTensor::load(txn, schema, dir)
                         .map_ok(Self::Sparse)
                         .await
@@ -269,7 +269,7 @@ impl Subject {
                 }
             }
             Schema::Value(value) => {
-                if let Some(file) = dir.get_file(txn.id(), &SUBJECT.into()).await? {
+                if let Some(file) = dir.get_file(*txn.id(), &SUBJECT.into()).await? {
                     Ok(Self::Value(file))
                 } else {
                     Self::create(Schema::Value(value), dir, *txn.id()).await
@@ -298,7 +298,7 @@ impl Subject {
                 State::Collection(Collection::Tensor(Tensor::Dense(backup))) => {
                     let file = txn
                         .context()
-                        .create_file_tmp(txn_id, TensorType::Dense)
+                        .create_file_unique(txn_id, TensorType::Dense)
                         .await?;
 
                     let backup = tc_transact::fs::CopyFrom::copy_from(backup, file, txn).await?;
@@ -313,7 +313,7 @@ impl Subject {
             #[cfg(feature = "tensor")]
             Self::Sparse(tensor) => match backup {
                 State::Collection(Collection::Tensor(Tensor::Sparse(backup))) => {
-                    let dir = txn.context().create_dir_tmp(txn_id).await?;
+                    let dir = txn.context().create_dir_unique(txn_id).await?;
                     let backup = tc_transact::fs::CopyFrom::copy_from(backup, dir, txn).await?;
                     tensor.restore(&backup, txn_id).await
                 }
@@ -376,7 +376,7 @@ impl de::FromStream for Subject {
             .map_err(de::Error::custom)
             .await?;
 
-        file.create_block(*txn.id(), SUBJECT.into(), value)
+        file.create_block(*txn.id(), SUBJECT.into(), value, BLOCK_SIZE)
             .map_err(de::Error::custom)
             .await?;
 
@@ -393,6 +393,7 @@ impl<'en> IntoView<'en, fs::Dir> for Subject {
         match self {
             Self::Value(file) => {
                 let value = file.read_block(*txn.id(), SUBJECT.into()).await?;
+
                 State::from(value.clone()).into_view(txn).await
             }
             Self::Table(table) => State::from(Table::Table(table)).into_view(txn).await,
