@@ -23,6 +23,7 @@ pub use after::After;
 pub use before::Before;
 pub use case::Case;
 pub use id::*;
+pub use new::New;
 pub use op::*;
 pub use r#if::IfRef;
 pub use r#while::While;
@@ -32,6 +33,7 @@ mod after;
 mod before;
 mod case;
 mod r#if;
+mod r#new;
 mod r#while;
 mod with;
 
@@ -78,6 +80,7 @@ pub enum RefType {
     Case,
     Id,
     If,
+    New,
     Op(OpRefType),
     While,
     With,
@@ -94,6 +97,7 @@ impl NativeClass for RefType {
                 "case" => Some(Self::Case),
                 "id" => Some(Self::Id),
                 "if" => Some(Self::If),
+                "new" => Some(Self::New),
                 "while" => Some(Self::While),
                 "with" => Some(Self::With),
                 _ => None,
@@ -112,6 +116,7 @@ impl NativeClass for RefType {
             Self::Case => "case",
             Self::Id => "id",
             Self::If => "if",
+            Self::New => "new",
             Self::Op(ort) => return ort.path(),
             Self::While => "while",
             Self::With => "with",
@@ -129,6 +134,7 @@ impl fmt::Display for RefType {
             Self::Case => f.write_str("Case"),
             Self::Id => f.write_str("Id"),
             Self::If => f.write_str("If"),
+            Self::New => f.write_str("New"),
             Self::Op(ort) => fmt::Display::fmt(ort, f),
             Self::While => f.write_str("While"),
             Self::With => f.write_str("With"),
@@ -144,6 +150,7 @@ pub enum TCRef {
     Case(Box<Case>),
     Id(IdRef),
     If(Box<IfRef>),
+    New(Box<New>),
     Op(OpRef),
     While(Box<While>),
     With(Box<With>),
@@ -159,6 +166,7 @@ impl Instance for TCRef {
             Self::Case(_) => RefType::Case,
             Self::Id(_) => RefType::Id,
             Self::If(_) => RefType::If,
+            Self::New(_) => RefType::New,
             Self::Op(op_ref) => RefType::Op(op_ref.class()),
             Self::While(_) => RefType::While,
             Self::With(_) => RefType::With,
@@ -187,6 +195,10 @@ impl Refer for TCRef {
                 let if_ref = if_ref.dereference_self(path);
                 Self::If(Box::new(if_ref))
             }
+            Self::New(new) => {
+                let new = new.dereference_self(path);
+                Self::New(Box::new(new))
+            }
             Self::Op(op_ref) => Self::Op(op_ref.dereference_self(path)),
             Self::While(while_ref) => {
                 let while_ref = while_ref.dereference_self(path);
@@ -206,6 +218,7 @@ impl Refer for TCRef {
             Self::Case(case) => case.is_conditional(),
             Self::Id(id_ref) => id_ref.is_conditional(),
             Self::If(if_ref) => if_ref.is_conditional(),
+            Self::New(new) => new.is_conditional(),
             Self::Op(op_ref) => op_ref.is_conditional(),
             Self::While(while_ref) => while_ref.is_conditional(),
             Self::With(with) => with.is_conditional(),
@@ -219,6 +232,7 @@ impl Refer for TCRef {
             Self::Case(case) => case.is_inter_service_write(cluster_path),
             Self::Id(id_ref) => id_ref.is_inter_service_write(cluster_path),
             Self::If(if_ref) => if_ref.is_inter_service_write(cluster_path),
+            Self::New(new) => new.is_inter_service_write(cluster_path),
             Self::Op(op_ref) => op_ref.is_inter_service_write(cluster_path),
             Self::While(while_ref) => while_ref.is_inter_service_write(cluster_path),
             Self::With(with) => with.is_inter_service_write(cluster_path),
@@ -244,6 +258,10 @@ impl Refer for TCRef {
                 let if_ref = if_ref.reference_self(path);
                 Self::If(Box::new(if_ref))
             }
+            Self::New(new) => {
+                let new = new.reference_self(path);
+                Self::New(Box::new(new))
+            }
             Self::Op(op_ref) => Self::Op(op_ref.reference_self(path)),
             Self::While(while_ref) => {
                 let while_ref = while_ref.reference_self(path);
@@ -263,6 +281,7 @@ impl Refer for TCRef {
             Self::Case(case) => case.requires(deps),
             Self::Id(id_ref) => id_ref.requires(deps),
             Self::If(if_ref) => if_ref.requires(deps),
+            Self::New(new) => new.requires(deps),
             Self::Op(op_ref) => op_ref.requires(deps),
             Self::While(while_ref) => while_ref.requires(deps),
             Self::With(with) => with.requires(deps),
@@ -282,6 +301,7 @@ impl Refer for TCRef {
             Self::After(after) => after.resolve(context, txn).await,
             Self::Before(before) => before.resolve(context, txn).await,
             Self::Id(id_ref) => id_ref.resolve(context, txn).await,
+            Self::New(new) => new.resolve(context, txn).await,
             Self::Op(op_ref) => op_ref.resolve(context, txn).await,
             Self::While(while_ref) => while_ref.resolve(context, txn).await,
             Self::With(with) => with.resolve(context, txn).await,
@@ -371,6 +391,13 @@ impl RefVisitor {
                     .map_ok(TCRef::If)
                     .await
             }
+            RefType::New => {
+                access
+                    .next_value(())
+                    .map_ok(Box::new)
+                    .map_ok(TCRef::New)
+                    .await
+            }
             RefType::Op(ort) => {
                 OpRefVisitor::visit_map_value(ort, access)
                     .map_ok(TCRef::Op)
@@ -466,6 +493,7 @@ impl<'en> ToStream<'en> for TCRef {
             Self::Before(before) => map.encode_value(before),
             Self::Case(case) => map.encode_value(case),
             Self::If(if_ref) => map.encode_value(if_ref),
+            Self::New(new) => map.encode_value(new),
             Self::While(while_ref) => map.encode_value(while_ref),
             Self::With(with) => map.encode_value(with),
         }?;
@@ -493,6 +521,7 @@ impl<'en> IntoStream<'en> for TCRef {
             Self::Before(before) => map.encode_value(before),
             Self::Case(case) => map.encode_value(case),
             Self::If(if_ref) => map.encode_value(if_ref),
+            Self::New(new) => map.encode_value(new),
             Self::While(while_ref) => map.encode_value(while_ref),
             Self::With(with) => map.encode_value(with),
         }?;
@@ -509,6 +538,7 @@ impl fmt::Debug for TCRef {
             Self::Case(case) => fmt::Debug::fmt(case, f),
             Self::Id(id_ref) => fmt::Debug::fmt(id_ref, f),
             Self::If(if_ref) => fmt::Debug::fmt(if_ref, f),
+            Self::New(new) => fmt::Debug::fmt(new, f),
             Self::Op(op_ref) => fmt::Debug::fmt(op_ref, f),
             Self::While(while_ref) => fmt::Debug::fmt(while_ref, f),
             Self::With(with) => fmt::Debug::fmt(with, f),
@@ -524,6 +554,7 @@ impl fmt::Display for TCRef {
             Self::Case(case) => fmt::Display::fmt(case, f),
             Self::Id(id_ref) => fmt::Display::fmt(id_ref, f),
             Self::If(if_ref) => fmt::Display::fmt(if_ref, f),
+            Self::New(new) => fmt::Display::fmt(new, f),
             Self::Op(op_ref) => fmt::Display::fmt(op_ref, f),
             Self::While(while_ref) => fmt::Display::fmt(while_ref, f),
             Self::With(with) => fmt::Display::fmt(with, f),
