@@ -26,14 +26,20 @@ mod request;
 mod server;
 
 struct Active {
+    workspace: fs::Dir,
     expires: NetworkTime,
     scope: Scope,
 }
 
 impl Active {
-    fn new(txn_id: &TxnId, expires: NetworkTime) -> Self {
+    fn new(txn_id: &TxnId, workspace: fs::Dir, expires: NetworkTime) -> Self {
         let scope = TCPathBuf::from(txn_id.to_id());
-        Self { expires, scope }
+
+        Self {
+            workspace,
+            expires,
+            scope,
+        }
     }
 
     fn expires(&self) -> &NetworkTime {
@@ -276,9 +282,17 @@ impl Transaction<fs::Dir> for Txn {
         })
     }
 
+    // TODO: rename to subcontext_unique
     async fn subcontext_tmp(&self) -> TCResult<Self> {
-        let id = self.dir.unique_id(self.id()).await?;
-        self.subcontext(id).await
+        self.dir
+            .create_dir_unique(*self.id())
+            .map_ok(|dir| Self {
+                active: self.active.clone(),
+                gateway: self.gateway.clone(),
+                request: self.request.clone(),
+                dir,
+            })
+            .await
     }
 }
 
