@@ -5,13 +5,13 @@ use std::convert::TryInto;
 use std::sync::Arc;
 use std::time::Duration;
 
-use futures::TryFutureExt;
+use freqfs::DirLock;
+use futures::future::TryFutureExt;
 use log::debug;
-use uplock::RwLock;
+use tokio::sync::RwLock;
 
 use tc_error::*;
 use tc_transact::fs::Dir;
-use tc_transact::Transact;
 
 use crate::fs;
 use crate::gateway::Gateway;
@@ -20,22 +20,20 @@ use super::request::*;
 use super::{Active, Txn, TxnId};
 
 const GRACE: Duration = Duration::from_secs(3);
-const INTERVAL: Duration = Duration::from_secs(1);
+const INTERVAL: Duration = Duration::from_millis(100);
 
 /// Server to keep track of the transactions currently active for this host.
 #[derive(Clone)]
 pub struct TxnServer {
-    active: RwLock<HashMap<TxnId, Arc<Active>>>,
-    workspace: fs::Dir,
+    active: Arc<RwLock<HashMap<TxnId, Arc<Active>>>>,
+    workspace: DirLock<fs::CacheBlock>,
 }
 
 impl TxnServer {
     /// Construct a new `TxnServer`.
-    pub async fn new(workspace: fs::Dir) -> Self {
-        let active = RwLock::new(HashMap::new());
-
+    pub async fn new(workspace: DirLock<fs::CacheBlock>) -> Self {
+        let active = Arc::new(RwLock::new(HashMap::new()));
         spawn_cleanup_thread(workspace.clone(), active.clone());
-
         Self { active, workspace }
     }
 
@@ -46,19 +44,23 @@ impl TxnServer {
         txn_id: TxnId,
         token: (String, Claims),
     ) -> TCResult<Txn> {
+        debug!("TxnServer::new_txn");
+
         let expires = token.1.expires().try_into()?;
-        let dir = self.txn_dir(txn_id).await?;
         let request = Request::new(txn_id, token.0, token.1);
         let mut active = self.active.write().await;
 
         match active.entry(txn_id) {
             Entry::Occupied(entry) => {
                 let active = entry.get();
+                let dir = active.workspace.create_dir_unique(txn_id).await?;
                 Ok(Txn::new(active.clone(), gateway, dir, request))
             }
             Entry::Vacant(entry) => {
-                let active = Arc::new(Active::new(&txn_id, expires));
-                let txn = Txn::new(active.clone(), gateway, self.workspace.clone(), request);
+                let workspace = self.txn_dir(txn_id).await?;
+                let dir = workspace.create_dir_unique(txn_id).await?;
+                let active = Arc::new(Active::new(&txn_id, workspace, expires));
+                let txn = Txn::new(active.clone(), gateway, dir, request);
                 entry.insert(active);
                 Ok(txn)
             }
@@ -67,11 +69,14 @@ impl TxnServer {
 
     /// Gracefully shut down this `TxnServer` by allowing all active transactions to drain.
     pub async fn shutdown(self) -> TCResult<()> {
+        debug!("TxnServer::shutdown");
+
         tokio::spawn(async move {
             let result = loop {
                 if self.active.read().await.is_empty() {
                     break TCResult::Ok(());
                 } else {
+                    debug!("TxnServer::shutdown pending active transactions");
                     tokio::time::sleep(tokio::time::Duration::from_millis(100)).await;
                 }
             };
@@ -83,12 +88,22 @@ impl TxnServer {
     }
 
     async fn txn_dir(&self, txn_id: TxnId) -> TCResult<fs::Dir> {
-        self.workspace.create_dir_tmp(txn_id).await
+        let mut workspace = self.workspace.write().await;
+        let cache = workspace
+            .create_dir(txn_id.to_string())
+            .map_err(fs::io_err)?;
+
+        fs::Dir::new(cache).await
     }
 }
 
-fn spawn_cleanup_thread(workspace: fs::Dir, active: RwLock<HashMap<TxnId, Arc<Active>>>) {
-    let mut interval = tokio::time::interval(tokio::time::Duration::from_secs(INTERVAL.as_secs()));
+fn spawn_cleanup_thread(
+    workspace: DirLock<fs::CacheBlock>,
+    active: Arc<RwLock<HashMap<TxnId, Arc<Active>>>>,
+) {
+    let mut interval = tokio::time::interval(tokio::time::Duration::from_millis(
+        INTERVAL.as_millis() as u64,
+    ));
 
     tokio::spawn(async move {
         loop {
@@ -98,27 +113,29 @@ fn spawn_cleanup_thread(workspace: fs::Dir, active: RwLock<HashMap<TxnId, Arc<Ac
     });
 }
 
-async fn cleanup(workspace: &fs::Dir, txn_pool: &RwLock<HashMap<TxnId, Arc<Active>>>) {
-    let expired = {
-        let now = Gateway::time();
-        let mut txn_pool = txn_pool.write().await;
-        let mut expired = Vec::with_capacity(txn_pool.len());
-        for (txn_id, txn) in txn_pool.iter() {
-            if txn.expires() + GRACE < now {
-                debug!("transaction {} has expired", txn_id);
-                expired.push(*txn_id);
+async fn cleanup(
+    workspace: &DirLock<fs::CacheBlock>,
+    txn_pool: &RwLock<HashMap<TxnId, Arc<Active>>>,
+) {
+    let now = Gateway::time();
+    let mut txn_pool = txn_pool.write().await;
+    let expired: Vec<TxnId> = txn_pool
+        .iter()
+        .filter_map(|(txn_id, active)| {
+            if active.expires() + GRACE < now {
+                Some(txn_id)
+            } else {
+                None
             }
-        }
-
-        for txn_id in &expired {
-            txn_pool.remove(txn_id);
-        }
-
-        expired
-    };
+        })
+        .cloned()
+        .collect();
 
+    let mut workspace = workspace.write().await;
     for txn_id in expired.into_iter() {
-        debug!("finalize expired transaction {}", txn_id);
-        workspace.finalize(&txn_id).await;
+        if let Some(_active) = txn_pool.remove(&txn_id) {
+            debug!("clean up txn {}", txn_id);
+            workspace.delete(txn_id.to_string());
+        }
     }
 }
